import { _decorator, Component, Vec3, systemEvent, SystemEvent, EventKeyboard, KeyCode } from "cc";

const { ccclass, property } = _decorator;

@ccclass("Camera")
export class Camera extends Component {
    @property({type: Number})
    moveSpeed: number = 5;

    private _isMovingLeft: boolean = false;
    private _isMovingRight: boolean = false;

    start () {
        systemEvent.on(SystemEvent.EventType.KEY_DOWN, this.onKeyDown, this);
        systemEvent.on(SystemEvent.EventType.KEY_UP, this.onKeyUp, this);
    }

    update (deltaTime: number) {
        let deltaMove = new Vec3();
        if (this._isMovingLeft) {
            deltaMove.x -= this.moveSpeed * deltaTime;
        }
        if (this._isMovingRight) {
            deltaMove.x += this.moveSpeed * deltaTime;
        }
        this.node.translate(deltaMove);
    }

    private onKeyDown (event: EventKeyboard) {
        switch (event.keyCode) {
            case KeyCode.KEY_A:
                this._isMovingLeft = true;
                break;
            case KeyCode.KEY_D:
                this._isMovingRight = true;
                break;
        }
    }

    private onKeyUp (event: EventKeyboard) {
        switch (event.keyCode) {
            case KeyCode.KEY_A:
                this._isMovingLeft = false;
                break;
            case KeyCode.KEY_D:
                this._isMovingRight = false;
                break;
        }
    }
}
