import { _decorator, Component, Node, Prefab, instantiate, Vec3 } from 'cc';
const { ccclass, property } = _decorator;

@ccclass('Main')
export class Main extends Component {
    @property(Prefab)
    carPrefab: Prefab = null!;

    private carList: { speed: number, carNode: Node }[] = [];
    private time: number = 0;
    private interval: number = 3;

    start() {
        this.addCar();
    }

    update(dt: number) {
        this.time += dt;
        if (this.time > this.interval) {
            this.addCar();
            this.time = 0;
        }

        this.moveCars(dt);
    }

    private addCar() {
        const carNode = instantiate(this.carPrefab);
        carNode.parent = this.node;
        const car = { speed: 5, carNode: carNode };
        this.carList.push(car);
    }

    private moveCars(dt: number) {
        const acceleration = 0.35;  // 加速度
        const drag = 0.05;  // 阻力系数

        for (let i = 0; i < this.carList.length; i++) {
            const car = this.carList[i];
            const carPosition = car.carNode.position;

            // 如果不是第一辆车，则判断与前一辆车的距离是否大于安全距离
            if (i > 0) {
                const frontCar = this.carList[i - 1];
                const frontCarPosition = frontCar.carNode.position;
                const distance = frontCarPosition.x - carPosition.x;
                if (distance < 5) {
                    car.speed = Math.max(0, car.speed - acceleration - drag);
                } else {
                    car.speed = Math.min(10, car.speed + acceleration - drag);
                }
            } else {
                // 第一辆车匀速低速行驶
                car.speed = Math.min(2, car.speed + acceleration - drag);
            }

            const newCarPosition = carPosition.add(new Vec3(car.speed * dt, 0, 0));
            car.carNode.setPosition(newCarPosition);
        }
    }

}
